import React, { useState, useEffect } from 'react';
import { GetStaticPaths, GetStaticProps } from 'next';
import Head from 'next/head';
import Image from 'next/image';
import { useRouter } from 'next/router';
import { FaStar, FaCheck, FaShoppingCart, FaHeart } from 'react-icons/fa';
import Layout from '@/components/Layout';
import Button from '@/components/common/Button';
import Breadcrumb from '@/components/Breadcrumb';
import ProductSlider from '@/components/product/ProductSlider';
import ReviewSection from '@/components/product/ReviewSection';
import { get } from '@/lib/api-client';
import { sanitizeHtml } from '@/lib/sanitize';
import { Product, Review } from '@/types/models';
import { useCart } from '@/hooks/useCart';
import { optimizeLCPImage } from '@/lib/imageOptimization';
import Meta from '@/components/common/Meta';
import ProductSchema from '@/components/schemas/ProductSchema';
import ErrorBoundary from '@/components/ErrorBoundary';
import styles from '@/styles/pages/ProductDetail.module.scss';

interface ProductDetailProps {
  product?: Product;
  relatedProducts?: Product[];
  reviews?: Review[];
}

const ProductDetail: React.FC<ProductDetailProps> = ({ 
  product: initialProduct,
  relatedProducts: initialRelatedProducts,
  reviews: initialReviews
}) => {
  const router = useRouter();
  const { addToCart } = useCart();
  
  const [product, setProduct] = useState<Product | undefined>(initialProduct);
  const [relatedProducts, setRelatedProducts] = useState<Product[]>(initialRelatedProducts || []);
  const [reviews, setReviews] = useState<Review[]>(initialReviews || []);
  const [loading, setLoading] = useState<boolean>(false);
  
  // Handle fallback loading state for ISR
  if (router.isFallback) {
    return (
      <Layout>
        <div className="container">
          <div className="loading-spinner">Loading...</div>
        </div>
      </Layout>
    );
  }
  
  // Handle case when product is not found
  if (!product) {
    return (
      <Layout>
        <div className="container">
          <div className="error-message">
            <h1>Product not found</h1>
            <p>The product you're looking for doesn't exist.</p>
            <Button href="/products" variant="primary">Browse Products</Button>
          </div>
        </div>
      </Layout>
    );
  }
  
  // Get optimized image props for the product
  const optimizedImage = optimizeLCPImage(product.image);
  
  // Breadcrumb items
  const breadcrumbItems = [
    { name: 'Trang chủ', path: '/' },
    { name: 'Sản phẩm', path: '/products' },
    { 
      name: product.category?.name || 'Danh mục', 
      path: `/category/${product.category?.slug}` 
    },
    { name: product.name, path: `/product/${product.slug}` }
  ];
  
  // Handle add to cart
  const handleAddToCart = () => {
    addToCart({
      _id: product._id,
      name: product.name,
      price: product.salePrice || product.price,
      image: product.image,
      slug: product.slug,
      quantity: 1
    });
  };
  
  // Generate structured data for SEO
  const structuredData = {
    '@context': 'https://schema.org/',
    '@type': 'Product',
    name: product.name,
    image: product.image,
    description: product.shortDescription,
    sku: product.sku,
    mpn: product._id,
    brand: {
      '@type': 'Brand',
      name: 'Premium Shop',
    },
    offers: {
      '@type': 'Offer',
      url: `${process.env.NEXT_PUBLIC_SITE_URL}/product/${product.slug}`,
      priceCurrency: 'VND',
      price: product.salePrice || product.price,
      priceValidUntil: product.saleEndDate || new Date(Date.now() + 30*24*60*60*1000).toISOString(),
      availability: product.inStock ? 'https://schema.org/InStock' : 'https://schema.org/OutOfStock',
      seller: {
        '@type': 'Organization',
        name: 'Premium Shop',
      },
    },
    aggregateRating: {
      '@type': 'AggregateRating',
      ratingValue: product.rating || 4.5,
      reviewCount: reviews.length || 5,
    },
  };
  
  return (
    <Layout>
      <Meta 
        title={product.name}
        description={product.shortDescription}
        ogType="product"
        ogImage={product.image}
        canonicalUrl={`${process.env.NEXT_PUBLIC_SITE_URL}/product/${product.slug}`}
        structuredData={structuredData}
      />
      
      <ProductSchema product={product} reviews={reviews} />
      
      <div className="container">
        <Breadcrumb items={breadcrumbItems} />
        
        <div className={styles.productDetail}>
          {/* Product Image */}
          <div className={styles.productImageContainer}>
            <Image
              src={product.image}
              alt={product.name}
              width={600}
              height={600}
              priority
              className={styles.productImage}
              sizes="(max-width: 768px) 100vw, 600px"
              srcSet={optimizedImage.srcset}
            />
          </div>
          
          {/* Product Info */}
          <div className={styles.productInfo}>
            <h1 className={styles.productTitle}>{product.name}</h1>
            
            {/* Product Rating */}
            <div className={styles.productRating}>
              {[...Array(5)].map((_, i) => (
                <FaStar
                  key={i}
                  className={i < Math.floor(product.rating || 0) ? styles.starActive : styles.star}
                />
              ))}
              <span>({reviews?.length || 0} đánh giá)</span>
            </div>
            
            {/* Product Price */}
            <div className={styles.productPrice}>
              {product.salePrice ? (
                <>
                  <span className={styles.salePrice}>
                    {product.salePrice.toLocaleString('vi-VN')}đ
                  </span>
                  <span className={styles.originalPrice}>
                    {product.price.toLocaleString('vi-VN')}đ
                  </span>
                  <span className={styles.discountBadge}>
                    {Math.round((1 - product.salePrice / product.price) * 100)}% Giảm
                  </span>
                </>
              ) : (
                <span className={styles.regularPrice}>
                  {product.price.toLocaleString('vi-VN')}đ
                </span>
              )}
            </div>
            
            {/* Short Description */}
            <div className={styles.shortDescription}>
              <p>{product.shortDescription}</p>
            </div>
            
            {/* Features */}
            <div className={styles.features}>
              <h3>Tính năng nổi bật</h3>
              <ul>
                {product.features?.map((feature, index) => (
                  <li key={index}>
                    <FaCheck /> {feature}
                  </li>
                ))}
              </ul>
            </div>
            
            {/* Actions */}
            <div className={styles.actions}>
              <Button 
                variant="primary" 
                size="lg" 
                icon={<FaShoppingCart />} 
                fullWidth 
                onClick={handleAddToCart}
                disabled={!product.inStock}
                testId="add-to-cart-button"
              >
                {product.inStock ? 'Thêm vào giỏ hàng' : 'Hết hàng'}
              </Button>
              
              <Button 
                variant="outline" 
                size="lg" 
                icon={<FaHeart />}
                testId="add-to-wishlist-button"
              >
                Yêu thích
              </Button>
            </div>
            
            {/* SKU, Category */}
            <div className={styles.meta}>
              {product.sku && (
                <p>
                  <span className={styles.metaLabel}>SKU:</span> {product.sku}
                </p>
              )}
              {product.category && (
                <p>
                  <span className={styles.metaLabel}>Danh mục:</span>{' '}
                  <a href={`/category/${product.category.slug}`}>
                    {product.category.name}
                  </a>
                </p>
              )}
            </div>
          </div>
        </div>
        
        {/* Product Description & Tabs */}
        <div className={styles.productTabs}>
          <ErrorBoundary>
            <div className={styles.tabContent}>
              <h2>Mô tả sản phẩm</h2>
              <div 
                className={styles.description}
                dangerouslySetInnerHTML={{ 
                  __html: sanitizeHtml(product.description || '') 
                }}
              />
            </div>
          </ErrorBoundary>
          
          {/* Reviews Section */}
          <ErrorBoundary>
            <ReviewSection 
              productId={product._id}
              initialReviews={reviews}
            />
          </ErrorBoundary>
        </div>
        
        {/* Related Products */}
        {relatedProducts.length > 0 && (
          <ErrorBoundary>
            <div className={styles.relatedProducts}>
              <h2>Sản phẩm liên quan</h2>
              <ProductSlider products={relatedProducts} />
            </div>
          </ErrorBoundary>
        )}
      </div>
    </Layout>
  );
};

export const getStaticPaths: GetStaticPaths = async () => {
  try {
    const response = await get<{ slugs: string[] }>('/api/products/slugs');
    
    const paths = response.data?.slugs.map((slug) => ({
      params: { slug }
    })) || [];
    
    return { 
      paths, 
      fallback: 'blocking' // ISR - Incremental Static Regeneration
    };
  } catch (error) {
    console.error('Error fetching product slugs:', error);
    return { paths: [], fallback: 'blocking' };
  }
};

export const getStaticProps: GetStaticProps<ProductDetailProps> = async ({ params }) => {
  try {
    const slug = params?.slug as string;
    
    // Fetch product data
    const response = await get<Product>(`/api/products/${slug}`);
    
    if (!response.success || !response.data) {
      return { notFound: true };
    }
    
    const product = response.data;
    
    // Fetch related products and reviews in parallel
    const [relatedResponse, reviewsResponse] = await Promise.all([
      get<Product[]>(`/api/products/related/${product._id}?limit=8`),
      get<Review[]>(`/api/reviews/product/${product._id}`)
    ]);
    
    return {
      props: {
        product,
        relatedProducts: relatedResponse.data || [],
        reviews: reviewsResponse.data || []
      },
      revalidate: 3600, // Revalidate every hour
    };
  } catch (error) {
    console.error('Error fetching product data:', error);
    return { notFound: true };
  }
};

export default ProductDetail;